package canary

import (
	"time"

	"code.cloudfoundry.org/lager"
)

type State int

const (
	NotUnhealthy State = iota
	Unhealthy
)

type StatefulStateMachine struct {
	State                 State
	OnBecomesUnhealthy    func(time.Time)
	OnBecomesNotUnhealthy func(time.Time)
	Logger                lager.Logger
}

func (m *StatefulStateMachine) BecomesUnhealthy(now time.Time) {
	if m.State == NotUnhealthy {
		m.Logger.Debug("StateMachine Transitioning to unhealthy", lager.Data{"now": now})
		m.OnBecomesUnhealthy(now)
	}

	m.Logger.Debug("StateMachine unhealthy", lager.Data{"now": now})

	m.State = Unhealthy
}

func (m *StatefulStateMachine) BecomesNotUnhealthy(now time.Time) {
	if m.State == Unhealthy {
		m.Logger.Debug("StateMachine Transitioning to not unhealthy", lager.Data{"now": now})
		m.OnBecomesNotUnhealthy(now)
	}

	m.Logger.Debug("StateMachine not unhealthy", lager.Data{"now": now})

	m.State = NotUnhealthy
}
